package zero1hd.polyjet.ui.builders;

import java.io.IOException;

import org.jaudiotagger.audio.AudioFileIO;
import org.jaudiotagger.audio.exceptions.CannotReadException;
import org.jaudiotagger.audio.exceptions.InvalidAudioFrameException;
import org.jaudiotagger.audio.exceptions.ReadOnlyFileException;
import org.jaudiotagger.audio.mp3.MP3File;
import org.jaudiotagger.audio.wav.WavTag;
import org.jaudiotagger.tag.FieldKey;
import org.jaudiotagger.tag.TagException;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.Preferences;
import com.badlogic.gdx.files.FileHandle;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.Pixmap;
import com.badlogic.gdx.graphics.Texture;
import com.badlogic.gdx.scenes.scene2d.ui.Button;
import com.badlogic.gdx.scenes.scene2d.ui.Image;
import com.badlogic.gdx.scenes.scene2d.ui.Label;
import com.badlogic.gdx.scenes.scene2d.ui.ScrollPane;
import com.badlogic.gdx.scenes.scene2d.ui.Skin;
import com.badlogic.gdx.scenes.scene2d.ui.Table;
import com.badlogic.gdx.utils.Align;
import com.badlogic.gdx.utils.Disposable;

import zero1hd.wavedecoder.WavInfo;

public class MusicSelectable extends Button implements Disposable {
	private boolean invalidMusic;
	private long durationInSeconds;
	private String songName;
	private Texture albumCover;
	WavInfo wavinfo;
	private String author;
	private int previousTop;
	private int ratedDifficulty;
	private byte[] albumWorkBytes;
	
	private Image imageIcon;
	private ScrollText displayName;
	private Label runTime;
	private Label authorLabel;
	private Label previousTopLabel;
	private Label ratedDifficultyLabel;
	private Skin skin;
	public MusicSelectable(FileHandle musicFile, Preferences musicData, final Skin skin, Texture defaultAlbumC) {
		super(skin, "info-button");
		this.skin = skin;
		
		setName(musicFile.name());
		
		this.albumCover = defaultAlbumC;
		
		if (musicFile.extension().toLowerCase().equals("mp3")) {
			MP3File mp3File;
			try {
				mp3File = new MP3File(musicFile.file());
				durationInSeconds = mp3File.getAudioHeader().getTrackLength();
				
				if (mp3File.getTag() != null && mp3File.getTag().getFirstArtwork() != null) {
					albumWorkBytes = mp3File.getTag().getFirstArtwork().getBinaryData();
				}
				
				songName = mp3File.getTag().getFirst(FieldKey.TITLE);
				author = mp3File.getTag().getFirst(FieldKey.ARTIST);
			} catch (IOException | TagException | ReadOnlyFileException | InvalidAudioFrameException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			
		} else {
			wavinfo = new WavInfo(musicFile.file());
			durationInSeconds = wavinfo.getDurationInSeconds();
			
			try {
				WavTag wavTag = (WavTag) AudioFileIO.read(wavinfo.getFile()).getTag();
				songName = wavTag.getFirst(FieldKey.TITLE);
				author = wavTag.getFirst(FieldKey.ARTIST);
			} catch (CannotReadException | IOException | TagException | ReadOnlyFileException
					| InvalidAudioFrameException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		if (durationInSeconds > 60 * 5) {
			invalidMusic = true;
		}
		
		if (songName == null || songName.isEmpty()) {
			songName = musicFile.nameWithoutExtension();
		}
		
		previousTop = musicData.getInteger(songName + ":previous top", -1);
		ratedDifficulty = musicData.getInteger(songName + ":difficulty", -1);
		
		if (author == null || author.isEmpty()) {
			author = "N/A";
		}
		
		defaults().pad(10f);
	}
	
	
	public void addInfoToPanel(ScrollPane scroller, float coverSize) {
		displayName = new ScrollText(songName, skin, true, scroller);
		
		defaults().align(Align.top);
		
		add(displayName).expandX().pad(0).fillX().padTop(10f).top().padBottom(10f);
		row();

		String formattedTime = "Run time: "+ String.valueOf(durationInSeconds/60) + ":";
		if (durationInSeconds - (durationInSeconds/60)*60 < 10) {
			formattedTime = formattedTime.concat("0");
		}
		
		Table songInfoTable = new Table();
		
		formattedTime = formattedTime.concat(String.valueOf(durationInSeconds - (durationInSeconds/60)*60));
		runTime = new Label(formattedTime, skin, "sub-font", skin.getColor("default"));
		songInfoTable.add(runTime).expandX().left();
		songInfoTable.row();
		
		authorLabel = new Label("Author: " + author, skin, "sub-font", skin.getColor("default"));
		songInfoTable.add(authorLabel).left();
		songInfoTable.row();
		
		previousTopLabel = new Label("High Score: " + (previousTop != -1 ? previousTop : "N/A"), skin, "sub-font", skin.getColor("default"));
		songInfoTable.add(previousTopLabel).left();
		songInfoTable.row();
		
		ratedDifficultyLabel = new Label("Difficulty: " + (ratedDifficulty != -1 ? ratedDifficulty : "N/A"), skin, "sub-font", skin.getColor("default"));
		songInfoTable.add(ratedDifficultyLabel).left();
		songInfoTable.row();
		
		row();
		add(songInfoTable).expandY().center();
		row();
		
		if (albumWorkBytes != null && !invalidMusic) {
			Pixmap albumArt = new Pixmap(albumWorkBytes, 0, albumWorkBytes.length);
			
			albumCover = new Texture(albumArt);
			imageIcon = new Image(albumCover);
			albumArt.dispose();
		} else {
			imageIcon = new Image(albumCover);
			Gdx.app.debug("UI", "album cover invalid or null for song: " + songName);
		}
		
		if (invalidMusic) {
			imageIcon.setColor(Color.RED);
		}
		add(imageIcon).expandY().center().pad(15f).size(coverSize);
	}
	
	public int getPreviousTop() {
		return previousTop;
	}
	
	public int getRatedDifficulty() {
		return ratedDifficulty;
	}
	
	public Texture getAlbumCover() {
		return albumCover;
	}
	
	public WavInfo getWavinfo() {
		return wavinfo;
	}
	
	@Override
	public void dispose() {
		albumCover.dispose();
	}
	
}
